@@ -3,7 +3,14 @@
  *  heap insertion would be O(log N).
  */
 #include <stdio.h>
+
+#if defined(_WIN32)
+#include <WinSock2.h>
+#define poll WSAPoll
+#pragma comment(lib, "ws2_32")
+#else
 #include <poll.h>
+#endif
 
 #include "duktape.h"
 #include "c_eventloop.h"
