@@ -71,7 +71,7 @@ int check_duktape_call_for_errors(int rc, duk_context* ctx)
          * access in a duk_safe_call() if it matters.
          */
         duk_get_prop_string(ctx, -1, "stack");
-        duk_console_log(DUK_CONSOLE_FLUSH | DUK_CONSOLE_TO_STDERR,
+        duk_console_log(ctx,DUK_CONSOLE_FLUSH | DUK_CONSOLE_TO_STDERR,
                         "error: %s\n", duk_safe_to_string(ctx, -1));
         duk_pop(ctx);
         return 0;
@@ -81,7 +81,7 @@ int check_duktape_call_for_errors(int rc, duk_context* ctx)
      * Error without a stack trace.
      * Non-Error value, coerce safely to string.
      */
-    duk_console_log(DUK_CONSOLE_FLUSH | DUK_CONSOLE_TO_STDERR,
+    duk_console_log(ctx,DUK_CONSOLE_FLUSH | DUK_CONSOLE_TO_STDERR,
                     "error: %s\n", duk_safe_to_string(ctx, -1));
     return 1;
 }
